package com.purui.ghip.service;

import com.alibaba.cloud.commons.lang.StringUtils;
import com.alibaba.fastjson2.JSON;
import com.purui.ghip.model.InterfaceDTO;
import com.purui.ghip.model.RequestDTO;
import com.purui.ghip.model.ServerConfigDTO;
import com.purui.ghip.utils.InterfaceCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.reactive.function.client.ExchangeStrategies;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.logging.Logger;

/**
 * 转发服务实现
 */
@Service
public class HealthService {

	@Autowired
	private WebClient.Builder webClientBuilder;

	@Autowired
	private InterfaceCache interfaceCache;

	Logger logger = Logger.getLogger(HealthService.class.getName());

	public Mono<String> health(HttpServletRequest request) throws Exception {
		return proxy(request);
	}

	// 从流中获取请求body参数
	private String getInputBody(HttpServletRequest request) {
		try {
			InputStream inputStream = request.getInputStream();
			ByteArrayOutputStream buffer = new ByteArrayOutputStream();
			int nRead;
			byte[] data = new byte[inputStream.available()]; // 16KB buffer
			while ((nRead = inputStream.read(data, 0, data.length)) != -1) {
				buffer.write(data, 0, nRead);
			}
			buffer.flush();
			return new String(buffer.toByteArray(), "UTF-8");
		} catch (Exception e) {
			System.out.println();
		}
		return null;
	}

	private Mono<String> proxy(HttpServletRequest request) throws Exception {
		// 获取请求body参数
		String requestBody = getInputBody(request);
//		logger.info("Request body: " + requestBody);
		if (StringUtils.isEmpty(requestBody)) {
			return Mono.just("parameters is empty.");
		}
		RequestDTO requestDTO = JSON.parseObject(requestBody, RequestDTO.class);
		// 根据请求URL查找对应的转发服务地址
		String hospCode = null;
		if (!CollectionUtils.isEmpty(requestDTO.getSession())) {
			hospCode = requestDTO.getSession().get(0).getHospCode();
		}
		// 从内存中获取接口配置
		ServerConfigDTO serverConfig = interfaceCache.getServerConfigs(hospCode);
		if (serverConfig == null) {
			return Mono.just("No interface configuration found.");
		}
		String serviceUrl = "/Service/sysInternalMutiple";
		String authorization = serverConfig.getProductAuthorization();
		String urlSchema = "http://%s:%s%s";
		// 默认写
		String targetUrl = String.format(urlSchema, serverConfig.getProductIP(), serverConfig.getProductPort(), serviceUrl);

		InterfaceDTO interfaceConfig = interfaceCache.getInterfaceConfigs(requestDTO.getCode());
		String serviceType = "";
		if (interfaceConfig != null) {
			if (StringUtils.isNotEmpty(interfaceConfig.getServiceUrl())) {
				serviceUrl = interfaceConfig.getServiceUrl();
			}
			// 如果有读写类型配置，直接用读写类型配置
			if (StringUtils.isNotEmpty(interfaceConfig.getServiceIP())) {
				authorization = interfaceConfig.getServiceAuth();
				targetUrl = String.format(urlSchema, interfaceConfig.getServiceIP(), interfaceConfig.getServicePort(), serviceUrl);
			} else {
				// 否则读取读写分离配置
				serviceType = interfaceConfig.getServiceType();
				// 只读
				if ("S".equalsIgnoreCase(serviceType)) {
					targetUrl = String.format(urlSchema, serverConfig.getQueryIP(), serverConfig.getQueryPort(), serviceUrl);
				}
			}
		}

		try {
			logger.info(String.format("Proxy %s request to %s", requestDTO.getCode(), targetUrl));
			// 使用WebClient进行异步转发请求
			String finalAuthorization = authorization;
			return webClientBuilder
					.exchangeStrategies(ExchangeStrategies
							.builder()
							.codecs(clientCodecConfigurer -> clientCodecConfigurer
									.defaultCodecs()
									.maxInMemorySize(1024 * 1024 * 10))
							.build())
					.build()
					.method(HttpMethod.valueOf(request.getMethod()))
					.uri(targetUrl)
					.headers(headers -> {
						headers.add(HttpHeaders.AUTHORIZATION, finalAuthorization);
					})
					.bodyValue(requestBody)
					.retrieve()
					.bodyToMono(String.class).onErrorComplete(
							throwable -> {
								logger.info("Proxy request failed: " + throwable.getMessage());
								return true;
							}
					);
		} catch (Exception e) {
			logger.info("Proxy request failed: " + e.getMessage());
			return Mono.just("访问iris服务异常!");
		}
	}
}
