package com.purui.ghip.utils;

import com.alibaba.cloud.commons.lang.StringUtils;
import com.alibaba.fastjson2.JSON;
import com.purui.ghip.config.DataConfig;
import com.purui.ghip.model.InterfaceDTO;
import com.purui.ghip.model.ResultDTO;
import com.purui.ghip.model.ServerConfigDTO;
import org.springframework.http.HttpMethod;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.client.ExchangeStrategies;
import org.springframework.web.reactive.function.client.WebClient;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;
import java.util.stream.Collectors;

/**
 * 接口缓存
 *
 * @author charlie
 * @date 2024/9/12
 */
@Component
@EnableScheduling
public class InterfaceCache {

	private static final String SERVER_CONFIG_KEY = "serverConfigs";
	private static final String INTERFACE_CONFIG_KEY = "interfaceConfigs";
	private static final String INTERFACE_UPDATE_KEY = "interfaceLastUpdate";
	Logger logger = Logger.getLogger(InterfaceCache.class.getName());

	@Resource
	private RedisUtil redisUtil;

	@Resource
	private WebClient.Builder webClientBuilder;

	@Resource
	private DataConfig dataConfig;

	@Scheduled(fixedRate = 60000 * 5) // 每5分钟更新一次
	public void updateCache() {
		String lastUpdateTime = redisUtil.get(INTERFACE_UPDATE_KEY);
		if (StringUtils.isEmpty(lastUpdateTime)) {
			lastUpdateTime = System.currentTimeMillis() + "";
		}
		else {
			// 如果最后更新时间小于2分钟，则不更新
			if (Long.parseLong(lastUpdateTime) > System.currentTimeMillis() - 120000) {
				return;
			}
		}

		if (StringUtils.isEmpty(dataConfig.getUrl())) {
			return;
		}

		// 服务器配置
		Map<String, String> serverConfigs = loadInterfaceServerConfig();
		if (serverConfigs != null && !serverConfigs.isEmpty()) {
			logger.info("load interface server config success.size:" + serverConfigs.size());
			redisUtil.setMap(SERVER_CONFIG_KEY, serverConfigs);
		}
		// 接口配置
		Map<String, String> interfaceConfigs = loadInterfaceConfigs();
		if (interfaceConfigs != null && !interfaceConfigs.isEmpty()) {
			logger.info("load interface config success. size:" + interfaceConfigs.size());
			redisUtil.setMap(INTERFACE_CONFIG_KEY, interfaceConfigs);
		}
		// 最后更新时间
		redisUtil.set(INTERFACE_UPDATE_KEY, lastUpdateTime);
	}

	public void initOnce() {
		// 初始化缓存，从数据库加载配置
		Map<Object, Object> interfaceConfigs = redisUtil.getMap(INTERFACE_CONFIG_KEY);
		// 加载配置
		if (interfaceConfigs == null || interfaceConfigs.isEmpty()) {
			updateCache();
		}
	}

	public ServerConfigDTO getServerConfigs(String hospCode) {
		if (StringUtils.isEmpty(hospCode)) {
			hospCode = dataConfig.getDefaultHospCode();
		}
		if (StringUtils.isEmpty(hospCode)) {
			return null;
		}
		String config = redisUtil.getMapValue(SERVER_CONFIG_KEY, hospCode);
		if (StringUtils.isEmpty(config)) {
			updateCache();  //right now
			return null;
		}
		return JSON.parseObject(config, ServerConfigDTO.class);
	}

	public InterfaceDTO getInterfaceConfigs(String code) {
		if (StringUtils.isEmpty(code)) {
			return null;
		}
		String config = redisUtil.getMapValue(INTERFACE_CONFIG_KEY, code);
		if (StringUtils.isEmpty(config)) {
			return null;
		}
		return JSON.parseObject(config, InterfaceDTO.class);
	}

	// 读写分离服务器配置信息
	private Map<String, String> loadInterfaceServerConfig() {
		String jsonRequest =
				"{\"params\":[{}],\"code\":\"012\",\"session\":[{}]}";
		String result = loadInterfaceConfigsFromExternalService(jsonRequest);
		// 解析JSON并转换为Map
		return parseJsonToServerMap(result);
	}

	private String loadInterfaceConfigsFromExternalService(String jsonRequest) {
		String jsonResponse = webClientBuilder
				.exchangeStrategies(ExchangeStrategies
						.builder()
						.codecs(clientCodecConfigurer -> clientCodecConfigurer
								.defaultCodecs()
								.maxInMemorySize(1024 * 1024 * 10))
						.build())
				.build()
				.method(HttpMethod.POST)
				.uri(dataConfig.getUrl())
				.headers(headers -> headers.add("Authorization", dataConfig.getAuthorization()))
				.bodyValue(jsonRequest)
				.retrieve()
				.bodyToMono(String.class)
				.onErrorComplete(err -> {
					logger.info("Proxy request failed: " + err.getMessage());
					return true;
				})
				.block();
		return jsonResponse;
	}

	// 接口信息
	private Map<String, String> loadInterfaceConfigs() {
		String jsonRequest =
				"{\"params\":[{}],\"code\":\"midpt008\",\"session\":[{}]}";
		String result = loadInterfaceConfigsFromExternalService(jsonRequest);

		return parseJsonToInterfaceMap(result);
	}

	private <T> List<T> parseJsonToResult(String json, Class<T> clazz) {
		if (!StringUtils.isEmpty(json)) {
			ResultDTO resultDTO = JSON.parseObject(json, ResultDTO.class);
			if (resultDTO != null) {
				return JSON.parseArray(JSON.toJSONString(resultDTO.getResult()), clazz);
			}
		}
		return null;
	}

	private Map<String, String> parseJsonToServerMap(String json) {
		Map<String, String> map = new HashMap<>(0);
		List<ServerConfigDTO> list = parseJsonToResult(json, ServerConfigDTO.class);
		if (list != null) {
			map = list
					.stream()
					.filter(x -> StringUtils.isNotEmpty(x.getHospCode()))
					.collect(Collectors.toMap(ServerConfigDTO::getHospCode, item -> JSON.toJSONString(item)));
		}
		return map;
	}

	private Map<String, String> parseJsonToInterfaceMap(String json) {
		Map<String, String> map = new HashMap<>(0);
		List<InterfaceDTO> list = parseJsonToResult(json, InterfaceDTO.class);
		if (list != null) {
			map = list
					.stream()
					.filter(x -> StringUtils.isNotEmpty(x.getCode()))
					.collect(Collectors.toMap(InterfaceDTO::getCode, item -> JSON.toJSONString(item)));
		}
		return map;
	}
}
